/*======================================================================*/
/*     Project: ExampleMemEval                                          */
/* Application: Example for resource analysis of Atmel programs         */
/*		  Note: on node 0 a repeater is needed to pass the signal to    */
/*		        the PC, and all unused targets must be quiet (they      */
/*		        must not send anything to the bus and keep it idle).    */
/*    Revision: 1.0                                                     */
/*        Date: 2006/10/23                                              */
/*      Author: Wilfried Elmenreich                                     */
/*  Adapted by: Alexander Koessler                                      */
/*   TU Vienna, Real Time Systems Group                                 */
/*======================================================================*/
#include "ExampleMemEval.h"



/*======================================================================*/
/* G L O B A L  V A R I A B L E S                                       */
/*======================================================================*/



/*======================================================================*/
/* I N T E R R U P T S                                                  */
/*======================================================================*/


/*======================================================================*/
/* P R O C E D U R E S                                                  */
/*======================================================================*/

uint16_t factorial(uint32_t k)
{
   if (k==0) return 1;
   return k*(factorial(k-1));	
}

void display_memory_stats(void)
{
   uint8_t *largest_begin,*largest_end;
   uint16_t largest_size;
  
   AnalyzeMemory(&largest_begin,&largest_size);
   largest_end = largest_begin + largest_size;
   
   SendString("Unused memory: 0x");

   SendHexByte((uint16_t)largest_begin >> 8); 
   SendHexByte((uint16_t)largest_begin & 0xFF);
   SendString(" - 0x");
   SendHexByte((uint16_t)largest_end >> 8); 
   SendHexByte((uint16_t)largest_end & 0xFF);
   SendString(" (");
   SendSize(largest_size);
   SendString(" Bytes)\r\n");

}


void StdOutInit(void){

  /* config Uart                                                        */
  MY_UART_CONTROL_A = MY_UART_CONTROL_A_INIT;
  MY_UART_CONTROL_B = MY_UART_CONTROL_B_INIT;
  MY_UART_CONTROL_C = MY_UART_CONTROL_C_INIT;
  MY_UART_BAUDRATE_H = MY_UART_BAUDRATE_H_INIT;
  MY_UART_BAUDRATE_L = MY_UART_BAUDRATE_L_INIT;

}



/*======================================================================*/
/* Main                                                                 */
/*======================================================================*/
int main(void){

  uint16_t alpha,omega,y;

  /* initializing the memory for evaluation                             */
  InitializeMemory();

  /* initializing the serial interface                                  */
  StdOutInit();
   
  SendString("\r\n\r\nInitialization complete!");
  SendString("\r\n========================\r\n");

  alpha = (uint16_t) EndOfUsedMem();
  omega = (uint16_t) GetStackPtr();
   
  SendString("End of statically used memory: 0x");
  SendHexByte(alpha >> 8); SendHexByte(alpha & 0xFF);
  SendString("\r\n");

  SendString("Stack pointer: 0x");
  SendHexByte(omega >> 8); SendHexByte(omega & 0xFF);
  SendString("\r\n\r\n");
  
  /* doing something to use memory                                      */
  display_memory_stats();
  
  y=factorial(3);

  display_memory_stats();
   
  y=factorial(20);

  display_memory_stats();
   
  y=factorial(25);

  display_memory_stats();

  return 0;
}
