/*======================================================================*/
/*     Project: ExampleMemEval                                          */
/* Application: Example for resource analysis of Atmel programs         */
/*		  Note: on node 0 a repeater is needed to pass the signal to    */
/*		        the PC, and all unused targets must be quiet (they      */
/*		        must not send anything to the bus and keep it idle).    */
/*    Revision: 1.0                                                     */
/*        Date: 2006/10/23                                              */
/*      Author: Wilfried Elmenreich                                     */
/*  Adapted by: Alexander Koessler                                      */
/*   TU Vienna, Real Time Systems Group                                 */
/*======================================================================*/
#ifndef __EXAMPLEMEMEVAL_H__
#define __EXAMPLEMEMEVAL_H__

#include <avr/io.h>
#include <stdio.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include "mem_eval.h"

#if defined (__AVR_ATmega128__)

#  define CPU_CLK           14745600
#  define BAUDRATE          19200
#  define BAUDRATE_MAGIC    ((CPU_CLK >> 4) / BAUDRATE - 1)

/* 19200 8N1, TX enable, RX disable                                     */
#  define MY_UART_CONTROL_A                 UCSR0A
#  define MY_UART_CONTROL_B                 UCSR0B
#  define MY_UART_CONTROL_C                 UCSR0C
#  define MY_UART_DATA_REGISTER             UDR0
#  define MY_UART_BAUDRATE_H                UBRR0H
#  define MY_UART_BAUDRATE_L                UBRR0L
#  define MY_UART_CONTROL_A_INIT            0x00
#  define MY_UART_CONTROL_B_INIT            0x08
#  define MY_UART_CONTROL_C_INIT            0x06
#  define MY_UART_BAUDRATE_H_INIT           (BAUDRATE_MAGIC >> 8)
#  define MY_UART_BAUDRATE_L_INIT           (BAUDRATE_MAGIC & 0xFF)
#  define MY_UART_CONTROL_UDRE              UDRE

#else
#  error "Wrong Target!\n"
#endif

#define SendChar(x) {                              \
  while(bit_is_clear(MY_UART_CONTROL_A,            \
                     MY_UART_CONTROL_UDRE));       \
  MY_UART_DATA_REGISTER = x;                       \
} 

#define SendHexByte(x) {                           \
  register uint8_t t;                              \
  t = '0' + (((uint8_t) (x)) >> 0x04);             \
  if(t > '9'){                                     \
    t += 'A' - '0' - 10;                           \
  }                                                \
  SendChar(t);                                     \
  t = '0' + (((uint8_t) (x)) & 0x0F);              \
  if(t > '9'){                                     \
    t += 'A' - '0' - 10;                           \
  }                                                \
  SendChar(t);                                     \
}

#define SendString(x) {                            \
  register char *buf=x;                            \
  while( *buf != '\0' ){                           \
    SendChar(*buf++);                              \
  }	                                           \
}

#define SendSize(x) {                              \
  register uint16_t t;                             \
  t = (((uint16_t)x) / 10000);                     \
  if( t == 0 ) /*not writing the first zero*/      \
  {                                                \
    SendChar(' ');                                 \
  }                                                \
  else                                             \
  {                                                \
    SendChar((uint8_t)t + '0');                    \
  }                                                \
  x = x - (t*10000);                               \
  t = ((uint8_t) (x / 1000));                      \
  SendChar((uint8_t)t + '0');                      \
  x = x - (t*1000);                                \
  t = ((uint8_t) (x / 100));                       \
  SendChar((uint8_t)t + '0');                      \
  x = x - (t*100);                                 \
  t = ((uint8_t) (x / 10));                        \
  SendChar((uint8_t)t + '0');                      \
  x = x - (t*10);                                  \
  SendChar((uint8_t)x + '0');                      \
}


#endif  /* ifndef __EXAMPLEMEMEVAL_H__ */
